#pragma once
#include "communication/bridge/connection_lifecycle/i_heartbeat_handler.h"
#include "robot/i_robot_io.h"
#include "util/i_watchdog_timer.h"
#include "util/time_source.h"
#include "util/toggle_action.h"
#include "domain/monitored_items.h"
#include <kr2_program_api/internals/api/console.h>
#include "spdlog/spdlog.h"

#include <functional>

namespace spin
{

    class HeartbeatHandler : public IHeartbeatHandler
    {
    public:
        HeartbeatHandler(std::shared_ptr<IRobotIO> robotIO, std::shared_ptr<IWatchdogTimer> watchdogTimer, std::shared_ptr<ITimeSource> timeSource);

        void setOnKickedCallback(std::function<void()> callback);
        void setOnDiedCallback(std::function<void()> callback) override;

        void handleHeartbeat(const MonitoredItems &monitoredItems) override;


    private:
        void restartWatchdogTimer();
        bool isOkayToWriteDigitalOutputNow();
        std::shared_ptr<IRobotIO> robotIO_;
        std::shared_ptr<ITimeSource> timeSource_;
        std::shared_ptr<IWatchdogTimer> watchdogTimer_;
        ToggleAction heartbeatAction_;
        static constexpr int heartbeat_pin = 6;
        static constexpr long watchdogWindowDuration_ms = 2000L;
        static constexpr long writeDigitalOutputPeriod_ms = 200;
        long lastTimeDigitalOutputWasWritten = 0;

        std::function<void()> onWatchdogKickedCallback_ = []()
        { SPDLOG_ERROR("HeartbeatHandler: onWatchdogKickedCallback_ not set"); };
    };

} // namespace spin